{-# LANGUAGE Arrows, CPP #-}
module Main where

#define ENABLE_VIEWER

import Control.Monad
import Control.Arrow
import Data.Maybe
import Data.Complex
import qualified Data.IntMap as IM
import System.IO
import Text.Printf
import System.Environment

import Phisics
import OrbitVM
import Controller
import SubmissionFile
#ifdef ENABLE_VIEWER
import SDLControl
import Control.Concurrent (forkIO, threadDelay, yield)
#endif

import Debug.Trace

import Data.IORef

-- -----------------------------------------------------------

main :: IO ()
main = do
  args <- getArgs
  let n = read $ head $ args ++ ["3001"]
  run n

run :: Int -> IO ()
run n = do
  vm <- newIOVM "../problems/bin3.obf"

#ifdef ENABLE_VIEWER
  oref <- newIORef Nothing
  let scale = case n of
                3001 -> 50000
                3002 -> 400000
                3003 -> 200000
                3004 -> 80000
                _ -> 100000
  forkIO $ startViewer 3 scale vm oref
#endif

  let trace t o = do
        when (t `mod` 1000 == 0) $ do
          let p1 = (- (o IM.! 0x2)) :+ (- (o IM.! 0x3))
              p2 = p1 + (o IM.! 0x4 :+ o IM.! 0x5)
          printf "t=%d: distance=%f\n" t (magnitude (p2 - p1))
          printf "self   = %s\n" (show p1) 
          printf "target = %s\n" (show p2)
          printf "selfRadius   = %f\n" (magnitude p1)
          printf "targetRadius = %f\n" (magnitude p2)
          hFlush stdout
          return ()
#ifdef ENABLE_VIEWER
        writeIORef oref (Just o)
        yield
#endif
  log <- controlWithTrace trace vm init c
  writeSubmissionFile (show n ++ ".osf") teamID n log

  where
    init = IM.fromList [(addrConf, fromIntegral n), (addrDVx, 0), (addrDVy, 0)]
    c = proc i -> do
          t <- time -< i
          v <- velocity -< i
          p <- pos -< i
          p2 <- targetSatelitePos -< i
          v2 <- targetSateliteVelocity -< i
          returnA -< 0
