{-# LANGUAGE Arrows, CPP #-}
module Main where

#define ENABLE_VIEWER

import Control.Monad
import Control.Arrow
import Data.Maybe
import Data.Complex
import qualified Data.IntMap as IM
import System.IO
import Text.Printf
import System.Environment

import Phisics
import OrbitVM
import Controller
import SubmissionFile
#ifdef ENABLE_VIEWER
import SDLControl
import Control.Concurrent (forkIO, threadDelay, yield)
#endif

import Debug.Trace

import Data.IORef

-- -----------------------------------------------------------

main :: IO ()
main = do
  args <- getArgs
  let n = read $ head $ args ++ ["1001"]
  run n

run :: Int -> IO ()
run n = do
  vm <- newIOVM "../problems/bin1.obf"

#ifdef ENABLE_VIEWER
  oref <- newIORef Nothing
  let scale = 500000
  forkIO $ startViewer 1 scale vm oref
#endif

  let trace t o = do
        when (t `mod` 1000 == 0) $ do
          let pos = (- (o IM.! addrSx)) :+ (- (o IM.! addrSy)) -- 地球からの相対座標
          printf "t=%d, radius=%f, targetRadius=%f\n" t (magnitude pos) (o IM.! addrTargetRadius)
          hFlush stdout
#ifdef ENABLE_VIEWER
        writeIORef oref (Just o)
        yield
#endif
  log <- controlWithTrace trace vm init c
  writeSubmissionFile (show n ++ ".osf") teamID n log

  where
    init = IM.fromList [(addrConf, fromIntegral n)]
    c = proc i -> do
          p <- pos -< i
          let r1 = magnitude p
          r2 <- targetRadius -< i
          v <- velocity -< i
          runTask_ task -< (v,r1,r2)
    task = do
      (v, r1, r2) <- snapT
      case v of
        Nothing -> output 0 >> task
        Just v -> hohmannTransfer v r1 r2 >> constT 0
