{-# LANGUAGE Arrows, CPP #-}
module Main where

-- # define ENABLE_VIEWER

import Control.Monad
import Control.Arrow
import Data.Maybe
import Data.Complex
import qualified Data.IntMap as IM
import System.IO
import Text.Printf
import System.Environment

import Phisics
import OrbitVM
import Controller
import SubmissionFile
#ifdef ENABLE_VIEWER
import SDLControl
import Control.Concurrent (forkIO, threadDelay, yield)
#endif

import Debug.Trace

import Data.IORef

-- -----------------------------------------------------------

main :: IO ()
main = do
  args <- getArgs
  let n = read $ head $ args ++ ["2001"]
  run n

run :: Int -> IO ()
run n = do
  vm <- newIOVM "../problems/bin2.obf"

#ifdef ENABLE_VIEWER
  oref <- newIORef Nothing
  let scale = case n of
                2001 -> 50000
                2002 -> 400000
                2003 -> 50000
                2004 -> 50000
                _ -> 100000
  forkIO $ startViewer 2 scale vm oref
#endif

  let trace t o = do
        when (t `mod` 1000 == 0) $ do
          let p1 = (- (o IM.! 0x2)) :+ (- (o IM.! 0x3))
              p2 = p1 + (o IM.! 0x4 :+ o IM.! 0x5)
          printf "t=%d: distance=%f\n" t (magnitude (p2 - p1))
          printf "self   = %s\n" (show p1) 
          printf "target = %s\n" (show p2)
          printf "selfRadius   = %f\n" (magnitude p1)
          printf "targetRadius = %f\n" (magnitude p2)
          hFlush stdout
          return ()
#ifdef ENABLE_VIEWER
        writeIORef oref (Just o)
        yield
#endif
  log <- controlWithTrace trace vm init c
  writeSubmissionFile (show n ++ ".osf") teamID n log

  where
    init = IM.fromList [(addrConf, fromIntegral n), (addrDVx, 0), (addrDVy, 0)]
    c = proc i -> do
          t <- time -< i
          v <- velocity -< i
          p <- pos -< i
          p2 <- targetSatelitePos -< i
          v2 <- targetSateliteVelocity -< i
          let
            r1 = magnitude p
            r2 = magnitude p2
            r3 = (r1 + r2) / 2
            t2 = pi * sqrt(r3**3 / mu)
            -- 2001,…,2004は角速度が負なので注意
            -- * どっち周りかをちゃんと計測する?
            -- * 自分の衛星と相手の衛星が逆回転してる場合のことを考える必要はある?
            w1 = - sqrt (mu / (r1**3))
            w2 = - sqrt (mu / (r2**3))
            theta1 = phase p
            theta2 = phase p2
            -- 現時点からのホーマン遷移の後の位相の差
            x' = (theta2 + t2 * w2) - (theta1 + signum w1 * pi)
            -- FIXME: 混乱中
            x = f (if w1 > 0 then x' else - x')
              where f a | a >= 2*pi = f (a - 2*pi)
                        | a < 0 = f (a + 2*pi)
                        | otherwise = a
            -- FIXME: 混乱中
            t' = if abs w1 > abs w2
                 then x / (abs w1 - abs w2)
                 else (2*pi - x) / (abs w2 - abs w1)
            start = t + round t'
          rec
            state <- fby S0 -< state'
            let (state', a) = (if t==1 then trace (printf "r1=%f r2=%f w1=%f w2=%f" r1 r2 w1 w2) else id) $
                  case state of
                    S0 ->
                      case v of
                        Nothing -> (S0, 0)
                        Just v -> trace ("scheduled: " ++ show start) $ (S1 start, 0)
                    S1 start'
                       | t==start' -> 
                           let dv1 = sqrt (mu / r1) * (sqrt (2 * r2 / (r1+r2)) - 1)
                               dv2 = sqrt (mu / r2) * (1 - sqrt (2 * r1 / (r1+r2)))
                           in trace (printf "entering Hohmann transfer orbit (t=%d): dv=%s" t (show dv1)) $
                              (S2 (t + round t2) dv2, mkPolar dv1 (phase (fromJust v)))
                       | start > t && start /= start' ->
                           trace (printf "rescheduled(t=%d): %d" t start) $
                           (S1 start, 0)
                       | otherwise -> (state, 0)
                    S2 end dv2
                       | t==end ->
                           trace (printf "leaving Hohmann transfer orbit (t=%d): dv=%s" t (show dv2)) $
                           (state, mkPolar dv2 (phase (fromJust v)))
                       | otherwise -> (state, 0)
          returnA -< a

data S
    = S0              -- 初期状態
    | S1 !Int         -- ホーマン遷移突入待ち
    | S2 !Int !Double -- ホーマン遷移終了待ち
