module Phisics where

import Data.Complex

type Pos = Complex Double
type Vel = Complex Double
type Acc = Complex Double

-- 重力定数
g :: Double
g = 6.67428 / 10**11

-- 地球の質量
me :: Double
me = 6.0 * 10**24

mu :: Double
mu = g * me

-- simulate x ys !! n でxのn秒後の状態を計算
simulate :: (Pos, Vel) -> [Acc] -> [(Pos, Vel)]
simulate x [] = [x]
simulate x (y:ys) = x : simulate (simulate1 x y) ys

-- simulate1 x y で x の1秒後の状態を計算
simulate1 :: (Pos, Vel) -> Acc -> (Pos, Vel)
simulate1 (s, v) dv = (s', v')
  where
    r = magnitude s
    s' = s + v * dt + (1/2) * (gg_s + dv) * dt**2
    v' = v + (dv + (gg_s + gg s') / 2) * dt
    gg_s = gg s
    gg s = mkPolar (g * me / (magnitude s)**2) (phase (- s))
    dt = 1
