module SubmissionFile where

import qualified Data.IntMap as IM
import qualified Data.IntSet as IS
import Data.Word
import Data.Maybe (fromMaybe)
import System.IO
import Foreign.Marshal (alloca)
import Foreign.Storable

import OrbitVM (Command, addrMax)

type TeamID = Int
type ScenarioID = Int
type TimeStep = Int
type Frame = (TimeStep, Command)

writeSubmissionFile :: FilePath -> TeamID -> ScenarioID -> [Frame] -> IO ()
writeSubmissionFile fname team scenario frames = do
  h <- openBinaryFile fname WriteMode
  alloca $ \pword -> alloca $ \pdouble -> do
    let wWord32 :: Word32 -> IO ()
        wWord32 x = poke pword x >> hPutBuf h pword (sizeOf x)
        wDouble :: Double -> IO ()
        wDouble x = poke pdouble x >> hPutBuf h pdouble (sizeOf x)
    wWord32 magic
    wWord32 $ fromIntegral teamID
    wWord32 $ fromIntegral scenario
    let f (tm, im) = do
          wWord32 $ fromIntegral tm
          wWord32 $ fromIntegral $ IM.size im
          mapM_ g $ IM.toAscList im
        g (addr, value) = do
          wWord32 $ fromIntegral addr
          wDouble value
    mapM_ f frames
  hClose h
  return ()

magic :: Word32
magic = 0xCAFEBABE

-- Team Sampou
teamID :: TeamID
teamID = 776

-- ----------------------------------------------------------------------

type Recorder = (Maybe Command, [Frame])

newRecorder :: Recorder
newRecorder = (Nothing, [])

cons :: Frame -> [Frame] -> [Frame]
cons x@(t, im) xs
    | IM.null im = xs
    | otherwise  = x : xs

record :: Recorder -> TimeStep -> Command -> Recorder
record r t im = seq t $ seq im $ 
    case r of
      (Nothing, xs) -> (Just im', cons (t, im') xs)
      (Just p, xs)  ->
          let im'' = IM.fromAscList
                     [ (a, y) | a <- IS.toAscList (IM.keysSet p `IS.union` IM.keysSet im')
                     , let x = fromMaybe 0 $ IM.lookup a p
                     , let y = fromMaybe 0 $ IM.lookup a im'
                     , x /= y
                     ]
          in (Just im', cons (t, im'') xs)
  where
    im' = IM.filter (0 /=) im

recordFin :: Recorder -> TimeStep -> [Frame]
recordFin (_, xs) t = reverse $ (t, IM.empty) : xs
